@@ -160,6 +160,8 @@ static gchar ArrayCallNames[MAX_CALL_NAMES][24] ={
 "GetRate"
 };
 
+void gok_scanner_input_motion_xevie (gint x, gint y);
+
 static xmlChar *gok_lang = NULL;
 
 /**
@@ -2193,7 +2195,6 @@ void gok_scanner_on_key_enter (GokKey* pKey)
 
 	gok_main_raise_window ();
 	gok_scanner_timer_start_dwell();
-		
 	gok_scanner_perform_effects (m_pEffectsOnKeyEnter);
 }
 
@@ -2319,8 +2320,8 @@ void gok_scanner_mouse_movement (gint x, gint y)
 {
 	m_MouseX = x;
 	m_MouseY = y;
-
 	gok_scanner_perform_effects (m_pEffectsMouseMovement);
+	gok_scanner_input_motion_xevie (x, y);
 }
 
 /**
@@ -2431,6 +2432,44 @@ void gok_scanner_input_motion_libusb (gint x, gint y)
 	gok_keyboard_paint_pointer (pKeyboard, pWindow, offset_x, offset_y);
 }
 
+/**
+* gok_scanner_input_motion_xevie
+* @x: Horizontal (first axis) input device motion component.
+* @y: Vertical (second axis)  input device motion component.
+*
+* Handler for input device motion events generated by an input device accessed through libusb. 
+**/
+void gok_scanner_input_motion_xevie (gint x, gint y)
+{
+	GokKey *pKey;
+	gint w, h;
+	static gint offset_x = 0, origin_x = 0;
+	static gint offset_y = 0, origin_y = 0;
+	GokKeyboard *pKeyboard = gok_main_get_current_keyboard ();
+	GtkWidget *pWindow = gok_main_get_main_window ();
+	
+	/* ignore motion events outside window */
+	gdk_drawable_get_size (pWindow->window, &w, &h);
+	gdk_window_get_position (pWindow->window, &offset_x, &offset_y);
+	gdk_window_get_root_origin (pWindow->window, &origin_x, &origin_y);
+	if (x < offset_x || x > offset_x+w ||
+	    y < offset_y || y > offset_y+h) {
+	  gok_scanner_on_key_leave (m_pContainingKey);
+	  return;
+	}
+   	pKey = gok_keyboard_find_key_at (pKeyboard, x-offset_x, y-offset_y, m_pContainingKey);
+	if (m_pContainingKey != pKey) {
+		gok_scanner_on_key_leave (m_pContainingKey);
+		if (pKey) {
+			gok_scanner_on_key_enter (pKey);
+		}
+		else if (m_pContainingKey) {
+		        gok_scanner_on_key_leave (m_pContainingKey);
+		}
+		m_pContainingKey = pKey;
+	}
+}
+
 
 /**
 * gok_scanner_on_timer1
@@ -3398,7 +3437,6 @@ gboolean
 gok_scanner_current_state_uses_corepointer (void)
 {
 	gboolean retval = FALSE;
-	GokAccessMethod *method;
 
 	if  (m_pEffectsLeftButtonDown ||
 	     m_pEffectsLeftButtonUp ||
@@ -3621,3 +3659,9 @@ gok_scanner_drop_refs (GokKey *pKey)
   if (m_pKeyEntered == pKey) m_pKeyEntered = NULL;
   if (m_pContainingKey == pKey) m_pContainingKey = NULL;
 }
+
+gboolean 
+gok_scanner_key_is_active (void)
+{
+  return m_pKeyEntered != NULL;
+}
